﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using java.util;
using com.ihr.xbrl.om;
using com.ihr.xbrl.om.xdt;
using java.io;
using java.net;
using com.ihr.xbrl.om.instance;
using com.ihr.xbrl.om.formula;
using javax.xml.@namespace;
using com.ihr.xbrl.om.taxonomy;

namespace PerformanceTestsStaticMode
{
    class Program
    {
        static void Main(string[] args)
        {
            System.Console.WriteLine("Starting...");
            // the main purpose of this demo is to show how to obtain dimensions in the DTS.

            // we will:
            // 1. open a taxonomy that contains dimensions.
            // 2. validate the document against the XBRL 2.1 and Dimensions specifications.
            // 3. loop over the dimensions and show information about it.

            // Let's define the properties of the DTSContainer first. By default, the DTSContainer will perform only XBRL 2.1 validation.
            // we can easily configure the processor to perform XBRL 2.1 and Dimensions validation.

            Properties dtsProps = new Properties();
            dtsProps.setProperty(DTSContainer.PROCESSORS_SEQUENCE,
                XBRLCoreProcessor.PROCESSORNAME + "," + XDTProcessor.PROCESSORNAME);

            // Hint: adding more processors to the sequence is possible (Formulas) it is also possible creating custom
            // processors for validation of project specific things that are not part of the XBRL and cannot be part
            // of the Formulas, Dimensions or core XBRL 2.1 and attach them to the sequence.

            // Now lets create the URL of the taxonomy file with dimensions
            // change this if the file is in another directory.
            // The selected instance document is using COREP taxonomies 1.3.
            File fTaxonomy = new File("TaxonomyFiles\\t-ca-2006-07-01.xsd");
            URI uTaxonomy = fTaxonomy.toURI(); // it is easier this way because URIs are more difficult to write when they are pointing to local files.

            // create the DTSContainer using indicated properties
            DateTime start = DateTime.Now;
            DTSContainer myDTS = DTSContainer.newEmptyContainer(dtsProps);
            DateTime stop = DateTime.Now;
            TimeSpan duration = new TimeSpan(stop.Ticks - start.Ticks);

            System.Console.WriteLine("Ready; "+ duration);
            System.Console.WriteLine("Press any key to start loading the instance...");
            System.Console.ReadKey(true);
            System.Console.WriteLine("Loading... this may take a while...");

            // note this cast is not safe and will fail if the document pointed to by the URL is a linkbase or taxonomy. But this is a demo.
            start = DateTime.Now;
            XBRLTaxonomy myTaxonomy = (XBRLTaxonomy)myDTS.load(uTaxonomy);
            stop = DateTime.Now;
            duration = new TimeSpan(stop.Ticks - start.Ticks);

            System.Console.WriteLine("Loaded "+duration+"; Press any key to start validating the taxonomy...");
            System.Console.ReadKey(true);
            System.Console.WriteLine("Validating...");

            // Now validate the instance document. This will loop all processor indicated in the PROCESSORS_SEQUENCE property
            // and will collect everything found (Errors, Inconsistencies, Assertions, Formula, ... into the XBRLValidationReport
            start = DateTime.Now;
            XBRLValidationReport report = myDTS.validate();
            stop = DateTime.Now;
            duration = new TimeSpan(stop.Ticks - start.Ticks);
            // in case of errors. we cannot continue. this call will thow the first error as an XBRLValidationExcecption and will do
            // nothing if the document is clean. Note we don't mind about inconsistencies at this stage. All of them are collected
            // inside the XBRLValidationReport
            report.throwExceptions();

            System.Console.WriteLine("Validated " + duration);
            System.Console.WriteLine("Press any key to start loading and validating the first instance document...");
            System.Console.ReadKey(true);

            // Now the DTS is ready to validate new instance documents under dynamic validation mode
            
            // Let's validate a small samle
            File fInstance1 = new File("TaxonomyFiles\\CA_instance.xml");
            URI uInstance1 = fInstance1.toURI(); // it is easier this way because URIs are more difficult to write when they are pointing to local files.

            // load instance from disk
            start = DateTime.Now;
            XBRLInstance instance1 = new XBRLInstance(new XMLDocumentURI(myDTS, null, uInstance1, null, false, null), false);
            if (!myDTS.isInstanceOfThisDTS(instance1,DTSContainer.FAST))
            {
                System.Console.WriteLine(fInstance1+" Cannot be validated for this DTS.");
            }

            XBRLValidationReport rep = myDTS.validate(instance1);
            stop = DateTime.Now;
            duration = new TimeSpan(stop.Ticks - start.Ticks);
            rep.throwExceptions();

            System.Console.WriteLine("Validated " + duration);
            System.Console.WriteLine("Press any key to start loading and validating the second instance document...");
            System.Console.ReadKey(true);

            // Let's validate a large sample
            File fInstance2 = new File("TaxonomyFiles\\CA_instanceLarge.xml");
            URI uInstance2 = fInstance2.toURI(); // it is easier this way because URIs are more difficult to write when they are pointing to local files.

            // load instance from disk
            start = DateTime.Now;
            XBRLInstance instance2 = new XBRLInstance(new XMLDocumentURI(myDTS, null, uInstance2, null, false, null), false);
            if (!myDTS.isInstanceOfThisDTS(instance2, DTSContainer.FAST))
            {
                System.Console.WriteLine(fInstance2 + " Cannot be validated for this DTS.");
            }

            XBRLValidationReport rep2 = myDTS.validate(instance2);
            stop = DateTime.Now;
            duration = new TimeSpan(stop.Ticks - start.Ticks);
            rep2.throwExceptions();

            System.Console.WriteLine("Validated " + duration);
            System.Console.WriteLine("Finished; Press any key to exit...");
            System.Console.ReadKey(true);
        }
    }
}
